@@ -6,10 +6,12 @@ local LogView = require "core.logview"
 
 
 local fullscreen = false
+local restore_title_view = false
 
 local function suggest_directory(text)
   text = common.home_expand(text)
-  return common.home_encode_list(text == "" and core.recent_projects or common.dir_path_suggest(text))
+  return common.home_encode_list((text == "" or text == common.home_expand(common.dirname(core.project_dir))) 
+    and core.recent_projects or common.dir_path_suggest(text))
 end
 
 command.add(nil, {
@@ -27,9 +29,12 @@ command.add(nil, {
 
   ["core:toggle-fullscreen"] = function()
     fullscreen = not fullscreen
+    if fullscreen then
+      restore_title_view = core.title_view.visible
+    end
     system.set_window_mode(fullscreen and "fullscreen" or "normal")
-    core.show_title_bar(not fullscreen)
-    core.title_view:configure_hit_test(not fullscreen)
+    core.show_title_bar(not fullscreen and restore_title_view)
+    core.title_view:configure_hit_test(not fullscreen and restore_title_view)
   end,
 
   ["core:reload-module"] = function()
@@ -66,8 +71,8 @@ command.add(nil, {
   end,
 
   ["core:find-file"] = function()
-    if core.project_files_limit then
-      return command.perform "core:open-file"
+    if not core.project_files_number() then
+       return command.perform "core:open-file"
     end
     local files = {}
     for dir, item in core.get_project_files() do
@@ -149,7 +154,7 @@ command.add(nil, {
   ["core:change-project-folder"] = function()
     local dirname = common.dirname(core.project_dir)
     if dirname then
-      core.command_view:set_text(common.home_encode(dirname) .. PATHSEP)
+      core.command_view:set_text(common.home_encode(dirname))
     end
     core.command_view:enter("Change Project Folder", function(text, item)
       text = system.absolute_path(common.home_expand(item and item.text or text))
@@ -159,14 +164,17 @@ command.add(nil, {
         core.error("Cannot open folder %q", text)
         return
       end
-      core.confirm_close_docs(core.docs, core.open_folder_project, text)
+      core.confirm_close_docs(core.docs, function(dirpath)
+        core.close_current_project()
+        core.open_folder_project(dirpath)
+      end, text)
     end, suggest_directory)
   end,
 
   ["core:open-project-folder"] = function()
     local dirname = common.dirname(core.project_dir)
     if dirname then
-      core.command_view:set_text(common.home_encode(dirname) .. PATHSEP)
+      core.command_view:set_text(common.home_encode(dirname))
     end
     core.command_view:enter("Open Project", function(text, item)
       text = common.home_expand(item and item.text or text)
@@ -191,8 +199,6 @@ command.add(nil, {
         return
       end
       core.add_project_directory(system.absolute_path(text))
-      -- TODO: add the name of directory to prioritize
-      core.reschedule_project_scan()
     end, suggest_directory)
   end,
 
