@@ -64,6 +64,22 @@ function core.set_project_dir(new_dir, change_project_fn)
   return false
 end
 
+function core.close_current_project()
+  -- When using system.unwatch_dir we need to pass the watch_id provided by dmon.
+  -- In reality when unwatching a directory the dmon library shifts the other watch_id
+  -- values so the actual watch_id changes. To workaround this problem we assume the
+  -- first watch_id is always 1 and the watch_id are continguous and we unwatch the
+  -- first watch_id repeateadly up to the number of watch_ids.
+  local watch_id_max = 0
+  for _, project_dir in ipairs(core.project_directories) do
+    if project_dir.watch_id and project_dir.watch_id > watch_id_max then
+      watch_id_max = project_dir.watch_id
+    end
+  end
+  for i = 1, watch_id_max do
+    system.unwatch_dir(1)
+  end
+end
 
 function core.open_folder_project(dir_path_abs)
   if core.set_project_dir(dir_path_abs, core.on_quit_project) then
@@ -199,12 +215,12 @@ local function file_search(files, info)
       inf = curr
     end
   end
-  repeat
+  while inf <= sup and not system.path_compare(filename, type, files[inf].filename, files[inf].type) do
     if files[inf].filename == filename then
       return inf, true
     end
     inf = inf + 1
-  until inf > sup or system.path_compare(filename, type, files[inf].filename, files[inf].type)
+  end
   return inf, false
 end
 
@@ -1164,8 +1180,8 @@ end
 
 
 -- no-op but can be overrided by plugins
-function core.on_dirmonitor_modify()
-end
+function core.on_dirmonitor_modify() end
+function core.on_dirmonitor_delete() end
 
 
 function core.on_dir_change(watch_id, action, filepath)
@@ -1174,6 +1190,7 @@ function core.on_dir_change(watch_id, action, filepath)
   core.dir_rescan_add_job(dir, filepath)
   if action == "delete" then
     project_scan_remove_file(dir, filepath)
+    core.on_dirmonitor_delete(dir, filepath)
   elseif action == "create" then
     project_scan_add_file(dir, filepath)
     core.on_dirmonitor_modify(dir, filepath);
